unit FANN;

interface

  {*******************************************************

   Only Delphi 6 and above supports variable arguments
   functions.
   If you are using a older version of Delphi comment the
   VARIABLE_ARGUMENTS directive.
   If you disable the VARIABLE_ARGUMENTS directive the
   following functions will not be available:
   fann_create_shortcut
   fann_create_sparse
   fann_create_standard

 2015_03_04 : Changed by Blue Hell:

   - changed DLL loading to be delayed, so it will only be loaded when it is actually needed.
   - changed layout

   ********************************************************}

{$DEFINE VARIABLE_ARGUMENTS}
  {*******************************************************

   If you want to use Fixed Fann or Double Fann please
   uncomment the corresponding definition.
   As default fann.pas uses the fannfloat dll.

   ********************************************************}
  // {$DEFINE FIXEDFANN} //Uncomment for fixed fann
  // {$DEFINE DOUBLEFANN} //Uncomment for double fann

const

{$IF Defined(FIXEDFANN)}
  FANN_DLL_FILE = 'fannfixed.dll';
{$ELSEIF Defined(DOUBLEFANN)}
  FANN_DLL_FILE = 'fanndouble.dll';
{$ELSE}
  FANN_DLL_FILE = 'fannfloat.dll';
{$IFEND}

type

{$IF Defined(FIXEDFANN)}
  fann_type = integer;
{$ELSEIF Defined(DOUBLEFANN)}
  fann_type = double;
{$ELSE}
  fann_type = Single;
{$IFEND}

  PFann_Type        = ^ fann_type;
  PPFann_Type       = ^ PFann_Type;
  Fann_Type_Array   = array[ 0 .. 65535] of fann_type;
  PFann_Type_Array  = ^ Fann_Type_Array;
  PPFann_Type_Array = array[ 0 .. 65535 ] of^Fann_Type_Array;

  (* MICROSOFT VC++ STDIO'S FILE DEFINITION*)
  _iobuf = packed record
    _ptr      : PAnsiChar;
    _cnt      : integer;
    _base     : PAnsiChar;
    _flag     : integer;
    _file     : integer;
    _charbuf  : integer;
    _bufsiz   : integer;
    _tmpfname : PAnsiChar;
  end;

  PFile = ^ TFile;
  TFile = _iobuf;

  PPFann_Neuron = ^ PFann_Neuron;
  PFann_Neuron  = ^ TFann_Neuron;

  TFann_Neuron = packed record
    first_con            : Cardinal;
    last_con             : Cardinal;
    sum                  : fann_type;
    value                : fann_type;
    activation_steepness : fann_type;
    activation_function  : Cardinal; // enum
  end;

  PFann_Layer = ^ TFann_Layer;

  TFann_Layer = packed record
    first_neuron : PFann_Neuron;
    last_neuron  : PFann_Neuron;
  end;

  PFann = ^ TFann;

  TFann = packed record
    errno_f                              : Cardinal;
    error_log                            : PFile;
    errstr                               : PAnsiChar;
    learning_rate                        : Single;
    learning_momentum                    : Single;
    connection_rate                      : Single;
    network_type                         : Cardinal; // ENUM
    first_layer                          : PFann_Layer;
    last_layer                           : PFann_Layer;
    total_neurons                        : Cardinal;
    num_input                            : Cardinal;
    num_output                           : Cardinal;
    weights                              : PFann_Type;
    connections                          : PPFann_Neuron;
    train_errors                         : PFann_Type;
    training_algorithm                   : Cardinal; // ENUM

{$IFDEF FIXEDFANN}
    decimal_point                        : Cardinal;
    multiplier                           : Cardinal;

    sigmoid_results                      : array[ 0 .. 5 ] of fann_type;
    sigmoid_values                       : array[ 0 .. 5 ] of fann_type;
    symmetric_results                    : array[ 0 .. 5 ] of fann_type;
    symmetric_values                     : array[ 0 .. 5 ] of fann_type;

{$ENDIF}
    total_connections                    : Cardinal;
    output                               : PFann_Type;
    num_MSE                              : Cardinal;
    MSE_value                            : Single;
    num_bit_fail                         : Cardinal;
    bit_fail_limit                       : fann_type;
    train_error_function                 : Cardinal;  // enum
    train_stop_function                  : Cardinal;  // enum
    callback                             : Pointer;   // TFANN_CALLBACK
    user_data                            : Pointer;
    cascade_output_change_fraction       : Single;
    cascade_output_stagnation_epochs     : Cardinal;
    cascade_candidate_change_fraction    : Single;
    cascade_candidate_stagnation_epochs  : Cardinal;
    cascade_best_candidate               : Cardinal;
    cascade_candidate_limit              : fann_type;
    cascade_weight_multiplier            : fann_type;
    cascade_max_out_epochs               : Cardinal;
    cascade_max_cand_epochs              : Cardinal;
    cascade_activation_functions         : PCardinal;
    cascade_activation_functions_count   : Cardinal;
    cascade_activation_steepnesses       : PFann_Type;
    cascade_activation_steepnesses_count : Cardinal;
    cascade_num_candidate_groups         : Cardinal;
    cascade_candidate_scores             : PFann_Type;
    total_neurons_allocated              : Cardinal;
    total_connections_allocated          : Cardinal;
    quickprop_decay                      : Single;
    quickprop_mu                         : Single;
    rprop_increase_factor                : Single;
    rprop_decrease_factor                : Single;
    rprop_delta_min                      : Single;
    rprop_delta_max                      : Single;
    rprop_delta_zero                     : Single;
    train_slopes                         : PFann_Type;
    prev_steps                           : PFann_Type;
    prev_train_slopes                    : PFann_Type;
    prev_weights_deltas                  : PFann_Type;

{$IFNDEF FIXEDFANN}
    scale_mean_in                        : pSingle;
    scale_deviation_in                   : pSingle;
    scale_new_min_in                     : pSingle;
    scale_factor_in                      : pSingle;
    scale_mean_out                       : pSingle;
    scale_deviation_out                  : pSingle;
    scale_new_min_out                    : pSingle;
    scale_factor_out                     : pSingle;
{$ENDIF}
  end;

  PFann_Train_Data = ^ TFann_Train_Data;

  TFann_Train_Data = packed record
    errno_f   : Cardinal;
    erro_log  : PFile;
    errstr    : PAnsiChar;
    num_data  : Cardinal;
    num_input : Cardinal;
    num_ouput : Cardinal;
    input     : PPFann_Type_Array;
    output    : PPFann_Type_Array;
  end;

  PFann_Connection =^TFann_Connection;

  TFann_Connection = packed record
    from_neuron : Cardinal;
    to_neuron   : Cardinal;
    weight      : fann_type;
  end;

  PFann_Error =^TFann_Error;

  TFann_Error = packed record
    errno_f   : Cardinal; // Enum
    error_log : PFile;
    errstr    : PAnsiChar;
  end;

  // _Fann_Train =
const

  FANN_TRAIN_INCREMENTAL = 0;
  FANN_TRAIN_BATCH       = 1;
  FANN_TRAIN_RPROP       = 2;
  FANN_TRAIN_QUICKPROP   = 3;


  // _Fann_Error_Func =

  FANN_ERRORFUNC_LINEAR = 0;
  FANN_ERRORFUNC_TANH   = 1;

  // _Fann_Activation_Func =
  FANN_LINEAR                     =  0;
  FANN_THRESHOLD                  =  1;
  FANN_THRESHOLD_SYMMETRIC        =  2;
  FANN_SIGMOID                    =  3;
  FANN_SIGMOID_STEPWISE           =  4;
  FANN_SIGMOID_SYMMETRIC          =  5;
  FANN_SIGMOID_SYMMETRIC_STEPWISE =  6;
  FANN_GAUSSIAN                   =  7;
  FANN_GAUSSIAN_SYMMETRIC         =  8;
  FANN_GAUSSIAN_STEPWISE          =  9;
  FANN_ELLIOT                     = 10;
  FANN_ELLIOT_SYMMETRIC           = 11;
  FANN_LINEAR_PIECE               = 12;
  FANN_LINEAR_PIECE_SYMMETRIC     = 13;
  FANN_SIN_SYMMETRIC              = 14;
  FANN_COS_SYMMETRIC              = 15;
  FANN_SIN                        = 16;
  FANN_COS                        = 17;

  // _Fann_ErroNo =
  FANN_E_NO_ERROR              =  0;
  FANN_E_CANT_OPEN_CONFIG_R    =  1;
  FANN_E_CANT_OPEN_CONFIG_W    =  2;
  FANN_E_WRONG_CONFIG_VERSION  =  3;
  FANN_E_CANT_READ_CONFIG      =  4;
  FANN_E_CANT_READ_NEURON      =  5;
  FANN_E_CANT_READ_CONNECTIONS =  6;
  FANN_E_WRONG_NUM_CONNECTIONS =  7;
  FANN_E_CANT_OPEN_TD_W        =  8;
  FANN_E_CANT_OPEN_TD_R        =  9;
  FANN_E_CANT_READ_TD          = 10;
  FANN_E_CANT_ALLOCATE_MEM     = 11;
  FANN_E_CANT_TRAIN_ACTIVATION = 12;
  FANN_E_CANT_USE_ACTIVATION   = 13;
  FANN_E_TRAIN_DATA_MISMATCH   = 14;
  FANN_E_CANT_USE_TRAIN_ALG    = 15;
  FANN_E_TRAIN_DATA_SUBSET     = 16;
  FANN_E_INDEX_OUT_OF_BOUND    = 17;
  FANN_E_SCALE_NOT_PRESENT     = 18;


  // _Fann_Stop_Func =

  FANN_STOPFUNC_MSE = 0;
  FANN_STOPFUNC_BIT = 1;

  // _Fann_Net_Type =
  FANN_NETTYPE_LAYER    = 0;
  FANN_NETTYPE_SHORTCUT = 1;

type

  TFann_CallBack = function(
    Ann                    : PFann;
    train                  : PFann_Train_Data;
    max_epochs             : Cardinal;
    epochs_between_reports : Cardinal;
    desired_error          : Single;
    epochs                 : Cardinal
 ): integer; cdecl;

  TUser_Function = procedure(
    num        : Cardinal;
    num_input  : Cardinal;
    num_output : Cardinal;
    input      : PFann_Type;
    output     : PFann_Type
 ); cdecl;

var

  FANN_ERRORFUNC_NAMES: array[ 0 .. 1] of string = (
    'FANN_ERRORFUNC_LINEAR',
    'FANN_ERRORFUNC_TANH'
  );

  FANN_TRAIN_NAMES: array[ 0 .. 3] of string = (
    'FANN_TRAIN_INCREMENTAL',
    'FANN_TRAIN_BATCH',
    'FANN_TRAIN_RPROP',
    'FANN_TRAIN_QUICKPROP'
  );

  FANN_ACTIVATIONFUNC_NAMES: array[ 0 .. 17] of string = (
    'FANN_LINEAR',
    'FANN_THRESHOLD',
    'FANN_THRESHOLD_SYMMETRIC',
    'FANN_SIGMOID',
    'FANN_SIGMOID_STEPWISE',
    'FANN_SIGMOID_SYMMETRIC',
    'FANN_SIGMOID_SYMMETRIC_STEPWISE',
    'FANN_GAUSSIAN',
    'FANN_GAUSSIAN_SYMMETRIC',
    'FANN_GAUSSIAN_STEPWISE',
    'FANN_ELLIOT',
    'FANN_ELLIOT_SYMMETRIC',
    'FANN_LINEAR_PIECE',
    'FANN_LINEAR_PIECE_SYMMETRIC',
    'FANN_SIN_SYMMETRIC',
    'FANN_COS_SYMMETRIC',
    'FANN_SIN',
    'FANN_COS'
  );

  FANN_STOPFUNC_NAMES: array[ 0 .. 1] of string = (
    'FANN_STOPFUNC_MSE',
    'FANN_STOPFUNC_BIT'
  );

  FANN_NETTYPE_NAMES: array[ 0 .. 1] of string = (
    'FANN_NETTYPE_LAYER',
    'FANN_NETTYPE_SHORTCUT'
  );


  // DECLARATIONS FROM FANN.H

{$IFDEF VARIABLE_ARGUMENTS}
  {

   ATTENTION!
   If your compilation breaks here maybe you are using a version of Delphi
   prior to 6. In this case you should comment the VARIABLE_ARGUMENTS define
   at the beginning of this file and live without this functions!
  }

  function  fann_create_standard( num_layers: Cardinal): PFann;                                                        cdecl; varargs;
  function  fann_create_sparse( connection_rate: Single; num_layers: Cardinal): PFann;                                 cdecl; varargs;
  function  fann_create_shortcut( connection_rate: Single): PFann;                                                     cdecl; varargs;

{$ENDIF}

  function  fann_create_standard_array( num_layers: Cardinal;const layers: PCardinal): PFann;                          cdecl;
  function  fann_create_sparse_array(
    connection_rate : Single;
    num_layers      : Cardinal;const
    layers          : PCardinal
  ) : PFann;                                                                                                           cdecl;

  function  fann_create_shortcut_array( num_layers: Cardinal;const layers: PCardinal): PFann;                          cdecl;
  procedure fann_destroy( Ann: PFann);                                                                                 cdecl;
  function  fann_run( Ann: PFann; input: PFann_Type): PFann_Type_Array;                                                cdecl;
  procedure fann_randomize_weights( Ann: PFann; Min_weight: fann_type; Max_weight: fann_type);                         cdecl;
  procedure fann_init_weights( Ann: PFann; train_data: PFann_Train_Data);                                              cdecl;
  procedure fann_print_connections( Ann: PFann);                                                                       cdecl;
  procedure fann_print_parameters( Ann: PFann);                                                                        cdecl;
  function  fann_get_num_input( Ann: PFann): Cardinal;                                                                 cdecl;
  function  fann_get_num_output( Ann: PFann): Cardinal;                                                                cdecl;
  function  fann_get_total_neurons( Ann: PFann): Cardinal;                                                             cdecl;
  function  fann_get_total_connections( Ann: PFann): Cardinal;                                                         cdecl;
  function  fann_get_network_type( Ann: PFann): Cardinal;                                                              cdecl;
  function  fann_get_connection_rate( Ann: PFann): Single;                                                             cdecl;
  function  fann_get_num_layers( Ann: PFann): Cardinal;                                                                cdecl;
  procedure fann_get_layer_array( Ann: PFann; layers: PCardinal);                                                      cdecl;
  procedure fann_get_bias_array( Ann: PFann; bias: PCardinal);                                                         cdecl;
  procedure fann_get_connection_array( Ann: PFann; connections: PFann_Connection);                                     cdecl;
  procedure fann_set_weight_array( Ann: PFann; connections: PFann_Connection; num_connection: Cardinal);               cdecl;
  procedure fann_set_weight( Ann: PFann; from_neuron: Cardinal; to_neuron: Cardinal; weight: fann_type);               cdecl;
  procedure fann_set_user_data( Ann: PFann; user_data: Pointer);                                                       cdecl;
  function  fann_get_user_data( Ann: PFann): Pointer;                                                                  cdecl;

{$IFDEF FIXEDFANN}
  function fann_get_decimal_point( Ann: PFann): Cardinal;                                                              cdecl;
  function fann_get_multiplier( Ann: PFann): Cardinal;                                                                 cdecl;
{$ENDIF}

  // END OF DECLARATIONS FROM FANN.H



  // DECLARATIONS FROM FANN_IO.H

  function  fann_create_from_file( const configuration_file: PAnsiChar): PFann;                                        cdecl;
  procedure fann_save( Ann: PFann;const configuration_file: PAnsiChar);                                                cdecl;
  function  fann_save_to_fixed( Ann: PFann;const configuration_file: PAnsiChar): integer;                              cdecl;

  // END OF DECLARATIONS FROM FANN_IO.H


  // DECLARATIONS FROM FANN_TRAIN.H

{$IFNDEF FIXEDFANN}
  procedure fann_train( Ann: PFann; input: PFann_Type; Desired_Output: PFann_Type);                                    cdecl;
{$ENDIF}

  function  fann_test( Ann: PFann; input: PFann_Type; Desired_Output: PFann_Type): PFann_Type_Array;                   cdecl;
  function  fann_get_MSE( Ann: PFann): Single;                                                                         cdecl;
  function  fann_get_bit_fail( Ann: PFann): Cardinal;                                                                  cdecl;
  procedure fann_reset_MSE( Ann: PFann);                                                                               cdecl;

{$IFNDEF FIXEDFANN}
  procedure fann_train_on_data(
    Ann                    : PFann;
    Data                   : PFann_Train_Data;
    max_epochs             : Cardinal;
    epochs_between_reports : Cardinal;
    desired_error          : Single
  );                                                                                                                   cdecl;
  procedure fann_train_on_file(
    Ann                    : PFann;
    Filename               : PAnsiChar;
    max_epochs             : Cardinal;
    epochs_between_reports : Cardinal;
    desired_error          : Single
  );                                                                                                                   cdecl;
  function  fann_train_epoch( Ann: PFann; Data: PFann_Train_Data): Single;                                             cdecl;
  function  fann_test_data( Ann: PFann; Data: PFann_Train_Data): Single;                                               cdecl;
{$ENDIF}

  function fann_read_train_from_file( const Filename: PAnsiChar): PFann_Train_Data;                                    cdecl;
  function fann_create_train_from_callback(
    num_data      : Cardinal;
    num_input     : Cardinal;
    num_output    : Cardinal;
    user_function : TUser_Function
  ): PFann_Train_Data;                                                                                                 cdecl;
  procedure fann_destroy_train( train_data: PFann_Train_Data);                                                         cdecl;
  procedure fann_shuffle_train_data( train_data: PFann_Train_Data);                                                    cdecl;
  procedure fann_scale_train( Ann: PFann; Data: PFann_Train_Data);                                                     cdecl;
  procedure fann_descale_train( Ann: PFann; Data: PFann_Train_Data);                                                   cdecl;
  function  fann_set_input_scaling_params(
    Ann            : PFann;
    const Data     : PFann_Train_Data;
    new_input_min  : Single;
    new_input_max  : Single
  ): integer;                                                                                                          cdecl;

  function  fann_set_output_scaling_params(
    Ann            : PFann;
    const Data     : PFann_Train_Data;
    new_output_min : Single;
    new_output_max : Single
  ): integer;                                                                                                          cdecl;
  function  fann_set_scaling_params(
    Ann: PFann;
    const Data     : PFann_Train_Data;
    new_input_min  : Single;
    new_input_max  : Single;
    new_output_min : Single;
    new_output_max : Single
  ): integer;                                                                                                          cdecl;
  function  fann_clear_scaling_params( Ann: PFann): integer;                                                           cdecl;
  procedure fann_scale_input( Ann: PFann; input_vector: PFann_Type);                                                   cdecl;
  procedure fann_scale_output( Ann: PFann; output_vector: PFann_Type);                                                 cdecl;
  procedure fann_descale_input( Ann: PFann; input_vector: PFann_Type);                                                 cdecl;
  procedure fann_descale_output( Ann: PFann; output_vector: PFann_Type);                                               cdecl;
  procedure fann_scale_input_train_data( train_data: PFann_Train_Data; new_min: fann_type; new_max: fann_type);        cdecl;
  procedure fann_scale_output_train_data( train_data: PFann_Train_Data; new_min: fann_type; new_max: fann_type);       cdecl;
  procedure fann_scale_train_data( train_data: PFann_Train_Data; new_min: fann_type; new_max: fann_type);              cdecl;
  function  fann_merge_train_data( Data1: PFann_Train_Data; Data2: PFann_Train_Data): PFann_Train_Data;                cdecl;
  function  fann_duplicate_train_data( Data: PFann_Train_Data): PFann_Train_Data;                                      cdecl;
  function  fann_subset_train_data( Data: PFann_Train_Data; pos: Cardinal; length: Cardinal): PFann_Train_Data;        cdecl;
  function  fann_length_train_data( Data: PFann_Train_Data): Cardinal;                                                 cdecl;
  function  fann_num_input_train_data( Data: PFann_Train_Data): Cardinal;                                              cdecl;
  function  fann_num_output_train_data( Data: PFann_Train_Data): Cardinal;                                             cdecl;
  function  fann_save_train( Data: PFann_Train_Data;const Filename: PAnsiChar): integer;                               cdecl;
  function  fann_save_train_to_fixed(
    Data           : PFann_Train_Data;
    const Filename : PAnsiChar;
    decimal_point  : Cardinal
  ) : integer;                                                                                                         cdecl;
  function  fann_get_training_algorithm( Ann: PFann): Cardinal;                                                        cdecl;
  procedure fann_set_training_algorithm( Ann: PFann; training_algorithm: Cardinal);                                    cdecl;
  function  fann_get_learning_rate( Ann: PFann): Single;                                                               cdecl;
  procedure fann_set_learning_rate( Ann: PFann; learning_rate: Single);                                                cdecl;
  function  fann_get_learning_momentum( Ann: PFann): Single;                                                           cdecl;
  procedure fann_set_learning_momentum( Ann: PFann; learning_momentum: Single);                                        cdecl;
  function  fann_get_activation_function( Ann: PFann; layer: integer; neuron: integer): Cardinal;                      cdecl; // ENUM
  procedure fann_set_activation_function( Ann: PFann; activation_function: Cardinal; layer: integer; neuron: integer); cdecl; // ENUM
  procedure fann_set_activation_function_layer( Ann: PFann; activation_function: Cardinal; layer: integer);            cdecl; // ENUM
  procedure fann_set_activation_function_hidden( Ann: PFann; activation_function: Cardinal);                           cdecl;
  procedure fann_set_activation_function_output( Ann: PFann; activation_function: Cardinal);                           cdecl;
  function  fann_get_activation_steepness( Ann: PFann; layer: integer; neuron: integer): fann_type;                    cdecl;
  procedure fann_set_activation_steepness( Ann: PFann; steepness: fann_type; layer: integer; neuron: integer);         cdecl;
  procedure fann_set_activation_steepness_layer( Ann: PFann; steepness: fann_type; layer: integer);                    cdecl;
  procedure fann_set_activation_steepness_hidden( Ann: PFann; steepness: fann_type);                                   cdecl;
  procedure fann_set_activation_steepness_output( Ann: PFann; steepness: fann_type);                                   cdecl;
  function  fann_get_train_error_function( Ann: PFann): Cardinal;                                                      cdecl;
  procedure fann_set_train_error_function( Ann: PFann; train_error_function: Cardinal);                                cdecl;
  function  fann_get_train_stop_function( Ann: PFann): Cardinal;                                                       cdecl;
  procedure fann_set_train_stop_function( Ann: PFann; train_stop_function: Cardinal);                                  cdecl;
  function  fann_get_bit_fail_limit( Ann: PFann): fann_type;                                                           cdecl;
  procedure fann_set_bit_fail_limit( Ann: PFann; bit_fail_limit: fann_type);                                           cdecl;
  procedure fann_set_callback( Ann: PFann; callback: TFann_CallBack);                                                  cdecl;
  function  fann_get_quickprop_decay( Ann: PFann): Single;                                                             cdecl;
  procedure fann_set_quickprop_decay( Ann: PFann; quickprop_decay: Single);                                            cdecl;
  function  fann_get_quickprop_mu( Ann: PFann): Single;                                                                cdecl;
  procedure fann_set_quickprop_mu( Ann: PFann; Mu: Single);                                                            cdecl;
  function  fann_get_rprop_increase_factor( Ann: PFann): Single;                                                       cdecl;
  procedure fann_set_rprop_increase_factor( Ann: PFann; rprop_increase_factor: Single);                                cdecl;
  function  fann_get_rprop_decrease_factor( Ann: PFann): Single;                                                       cdecl;
  procedure fann_set_rprop_decrease_factor( Ann: PFann; rprop_decrease_factor: Single);                                cdecl;
  function  fann_get_rprop_delta_min( Ann: PFann): Single;                                                             cdecl;
  procedure fann_set_rprop_delta_min( Ann: PFann; rprop_delta_min: Single);                                            cdecl;
  function  fann_get_rprop_delta_max( Ann: PFann): Single;                                                             cdecl;
  procedure fann_set_rprop_delta_max( Ann: PFann; rprop_delta_max: Single);                                            cdecl;
  function  fann_get_rprop_delta_zero( Ann: PFann): Single;                                                            cdecl;
  procedure fann_set_rprop_delta_zero( Ann: PFann; rprop_delta_zero: Single);                                          cdecl;

  // END OF DECLARATIONS OF FANN_TRAIN.H


  // DECLARATIONS OF FANN_ERROR.H

  procedure fann_set_error_log( errdat: PFann_Error; Log_File: PFile);                                                 cdecl;
  function  fann_get_errno( errdat: PFann_Error): Cardinal;                                                            cdecl;
  procedure fann_reset_errno( errdat: PFann_Error);                                                                    cdecl;
  procedure fann_reset_errstr( errdat: PFann_Error);                                                                   cdecl;
  function  fann_get_errstr( errdat: PFann_Error): PAnsiChar;                                                          cdecl;
  procedure fann_print_error( errdat: PFann_Error);                                                                    cdecl;

  // END OF DECLARATIONS OF FANN_ERROR

  // DECLARATIONS OF FANN_CASCADE.H

  procedure fann_cascadetrain_on_data(
    Ann: PFann;
    Data                    : PFann_Train_Data;
    max_neurons             : Cardinal;
    neurons_between_reports : Cardinal;
    desired_error           : Single
  );                                                                                                                   cdecl;
  procedure fann_cascadetrain_on_file(
    Ann                     : PFann;
    const Filename          : PAnsiChar;
    max_neurons             : Cardinal;
    neurons_between_reports : Cardinal;
    desired_error           : Single
  );                                                                                                                   cdecl;
  function  fann_get_cascade_output_change_fraction( Ann: PFann): Single;                                              cdecl;
  procedure fann_set_cascade_output_change_fraction( Ann: PFann; cascade_output_change_fraction: Single);              cdecl;
  function  fann_get_cascade_output_stagnation_epochs( Ann: PFann): Cardinal;                                          cdecl;
  procedure fann_set_cascade_output_stagnation_epochs( Ann: PFann; cascade_output_stagnation_epochs: Cardinal);        cdecl;
  function  fann_get_cascade_candidate_change_fraction( Ann: PFann): Single;                                           cdecl;
  procedure fann_set_cascade_candidate_change_fraction( Ann: PFann; cascade_candidate_change_fraction: Single);        cdecl;
  function  fann_get_cascade_candidate_stagnation_epochs( Ann: PFann): Cardinal;                                       cdecl;
  procedure fann_set_cascade_candidate_stagnation_epochs( Ann: PFann; cascade_candidate_stagnation_epochs: Cardinal);  cdecl;
  function  fann_get_cascade_weight_multiplier( Ann: PFann): fann_type;                                                cdecl;
  procedure fann_set_cascade_weight_multiplier( Ann: PFann; cascade_weight_multiplier: fann_type);                     cdecl;
  function  fann_get_cascade_candidate_limit( Ann: PFann): fann_type;                                                  cdecl;
  procedure fann_set_cascade_candidate_limit( Ann: PFann; cascade_candidate_limit: fann_type);                         cdecl;
  function  fann_get_cascade_max_out_epochs( Ann: PFann): Cardinal;                                                    cdecl;
  procedure fann_set_cascade_max_out_epochs( Ann: PFann; cascade_max_out_epochs: Cardinal);                            cdecl;
  function  fann_get_cascade_max_cand_epochs( Ann: PFann): Cardinal;                                                   cdecl;
  procedure fann_set_cascade_max_cand_epochs( Ann: PFann; cascade_max_cand_epochs: Cardinal);                          cdecl;
  function  fann_get_cascade_num_candidates( Ann: PFann): Cardinal;                                                    cdecl;
  function  fann_get_cascade_activation_functions_count( Ann: PFann): Cardinal;                                        cdecl;
  function  fann_get_cascade_activation_functions( Ann: PFann): PCardinal;                                             cdecl;
  procedure fann_set_cascade_activation_functions(
    Ann                                : PFann;
    cascade_activation_functions       : PCardinal;
    cascade_activation_functions_count : Cardinal
  );                                                                                                                   cdecl;
  function  fann_get_cascade_activation_steepnesses_count( Ann: PFann): Cardinal;                                      cdecl;
  function  fann_get_cascade_activation_steepnesses( Ann: PFann): PFann_Type;                                          cdecl;
  procedure fann_set_cascade_activation_steepnesses(
    Ann                                  : PFann;
    cascade_activation_steepnesses       : PFann_Type;
    cascade_activation_steepnesses_count : Cardinal
  );                                                                                                                   cdecl;
  function  fann_get_cascade_num_candidate_groups( Ann: PFann): Cardinal;                                              cdecl;
  procedure fann_set_cascade_num_candidate_groups( Ann: PFann; cascade_num_candidate_groups: Cardinal);                cdecl;

  // END OF DECLARATIONS OF FANN_CASCADE.H



implementation



{$IFDEF VARIABLE_ARGUMENTS}
  function  fann_create_standard;                          external FANN_DLL_FILE delayed;
{$ENDIF}

  function  fann_create_standard_array;                    external FANN_DLL_FILE delayed;

{$IFDEF VARIABLE_ARGUMENTS}
  function  fann_create_sparse;                            external FANN_DLL_FILE delayed;
{$ENDIF}

  function  fann_create_sparse_array;                      external FANN_DLL_FILE delayed;

{$IFDEF VARIABLE_ARGUMENTS}
  function  fann_create_shortcut;                          external FANN_DLL_FILE delayed;
{$ENDIF}

  function  fann_create_shortcut_array;                    external FANN_DLL_FILE delayed;

  procedure fann_destroy;                                  external FANN_DLL_FILE delayed;
  function  fann_run;                                      external FANN_DLL_FILE delayed;
  procedure fann_randomize_weights;                        external FANN_DLL_FILE delayed;
  procedure fann_init_weights;                             external FANN_DLL_FILE delayed;
  procedure fann_print_connections;                        external FANN_DLL_FILE delayed;
  procedure fann_print_parameters;                         external FANN_DLL_FILE delayed;
  function  fann_get_num_input;                            external FANN_DLL_FILE delayed;
  function  fann_get_num_output;                           external FANN_DLL_FILE delayed;
  function  fann_get_total_neurons;                        external FANN_DLL_FILE delayed;
  function  fann_get_total_connections;                    external FANN_DLL_FILE delayed;
  function  fann_get_network_type;                         external FANN_DLL_FILE delayed;
  function  fann_get_connection_rate;                      external FANN_DLL_FILE delayed;
  function  fann_get_num_layers;                           external FANN_DLL_FILE delayed;
  procedure fann_get_layer_array;                          external FANN_DLL_FILE delayed;
  procedure fann_get_bias_array;                           external FANN_DLL_FILE delayed;
  procedure fann_get_connection_array;                     external FANN_DLL_FILE delayed;
  procedure fann_set_weight_array;                         external FANN_DLL_FILE delayed;
  procedure fann_set_weight;                               external FANN_DLL_FILE delayed;
  procedure fann_set_user_data;                            external FANN_DLL_FILE delayed;
  function  fann_get_user_data;                            external FANN_DLL_FILE delayed;

{$IFDEF FIXEDFANN}
  function  fann_get_decimal_point;                        external FANN_DLL_FILE delayed;
  function  fann_get_multiplier;                           external FANN_DLL_FILE delayed;
{$ENDIF}

  // END OF DECLARATIONS FROM FANN.H


  // DECLARATIONS FROM FANN_IO.H

  function  fann_create_from_file;                         external FANN_DLL_FILE delayed;
  procedure fann_save;                                     external FANN_DLL_FILE delayed;
  function  fann_save_to_fixed;                            external FANN_DLL_FILE delayed;

  // END OF DECLARATIONS FROM FANN_IO.H


  // DECLARATIONS FROM FANN_TRAIN.H

{$IFNDEF FIXEDFANN}
  procedure fann_train;                                    external FANN_DLL_FILE delayed;
{$ENDIF}

  function  fann_test;                                     external FANN_DLL_FILE delayed;
  function  fann_get_MSE;                                  external FANN_DLL_FILE delayed;
  function  fann_get_bit_fail;                             external FANN_DLL_FILE delayed;
  procedure fann_reset_MSE;                                external FANN_DLL_FILE delayed;

{$IFNDEF FIXEDFANN}
  procedure fann_train_on_data;                            external FANN_DLL_FILE delayed;
  procedure fann_train_on_file;                            external FANN_DLL_FILE delayed;
  function  fann_train_epoch;                              external FANN_DLL_FILE delayed;
  function  fann_test_data;                                external FANN_DLL_FILE delayed;
{$ENDIF}

  function  fann_read_train_from_file;                     external FANN_DLL_FILE delayed;
  function  fann_create_train_from_callback;               external FANN_DLL_FILE delayed;
  procedure fann_destroy_train;                            external FANN_DLL_FILE delayed;
  procedure fann_shuffle_train_data;                       external FANN_DLL_FILE delayed;
  procedure fann_scale_train;                              external FANN_DLL_FILE delayed;
  procedure fann_descale_train;                            external FANN_DLL_FILE delayed;
  function  fann_set_input_scaling_params;                 external FANN_DLL_FILE delayed;
  function  fann_set_output_scaling_params;                external FANN_DLL_FILE delayed;
  function  fann_set_scaling_params;                       external FANN_DLL_FILE delayed;
  function  fann_clear_scaling_params;                     external FANN_DLL_FILE delayed;
  procedure fann_scale_input;                              external FANN_DLL_FILE delayed;
  procedure fann_scale_output;                             external FANN_DLL_FILE delayed;
  procedure fann_descale_input;                            external FANN_DLL_FILE delayed;
  procedure fann_descale_output;                           external FANN_DLL_FILE delayed;
  procedure fann_scale_input_train_data;                   external FANN_DLL_FILE delayed;
  procedure fann_scale_output_train_data;                  external FANN_DLL_FILE delayed;
  procedure fann_scale_train_data;                         external FANN_DLL_FILE delayed;
  function  fann_merge_train_data;                         external FANN_DLL_FILE delayed;
  function  fann_duplicate_train_data;                     external FANN_DLL_FILE delayed;
  function  fann_subset_train_data;                        external FANN_DLL_FILE delayed;
  function  fann_length_train_data;                        external FANN_DLL_FILE delayed;
  function  fann_num_input_train_data;                     external FANN_DLL_FILE delayed;
  function  fann_num_output_train_data;                    external FANN_DLL_FILE delayed;
  function  fann_save_train;                               external FANN_DLL_FILE delayed;
  function  fann_save_train_to_fixed;                      external FANN_DLL_FILE delayed;
  function  fann_get_training_algorithm;                   external FANN_DLL_FILE delayed;
  procedure fann_set_training_algorithm;                   external FANN_DLL_FILE delayed;
  function  fann_get_learning_rate;                        external FANN_DLL_FILE delayed;
  procedure fann_set_learning_rate;                        external FANN_DLL_FILE delayed;
  function  fann_get_learning_momentum;                    external FANN_DLL_FILE delayed;
  procedure fann_set_learning_momentum;                    external FANN_DLL_FILE delayed;
  function  fann_get_activation_function;                  external FANN_DLL_FILE delayed; // ENUM
  procedure fann_set_activation_function;                  external FANN_DLL_FILE delayed; // ENUM
  procedure fann_set_activation_function_layer;            external FANN_DLL_FILE delayed; // ENUM
  procedure fann_set_activation_function_hidden;           external FANN_DLL_FILE delayed;
  procedure fann_set_activation_function_output;           external FANN_DLL_FILE delayed;
  function  fann_get_activation_steepness;                 external FANN_DLL_FILE delayed;
  procedure fann_set_activation_steepness;                 external FANN_DLL_FILE delayed;
  procedure fann_set_activation_steepness_layer;           external FANN_DLL_FILE delayed;
  procedure fann_set_activation_steepness_hidden;          external FANN_DLL_FILE delayed;
  procedure fann_set_activation_steepness_output;          external FANN_DLL_FILE delayed;
  function  fann_get_train_error_function;                 external FANN_DLL_FILE delayed;
  procedure fann_set_train_error_function;                 external FANN_DLL_FILE delayed;
  function  fann_get_train_stop_function;                  external FANN_DLL_FILE delayed;
  procedure fann_set_train_stop_function;                  external FANN_DLL_FILE delayed;
  function  fann_get_bit_fail_limit;                       external FANN_DLL_FILE delayed;
  procedure fann_set_bit_fail_limit;                       external FANN_DLL_FILE delayed;
  procedure fann_set_callback;                             external FANN_DLL_FILE delayed;
  function  fann_get_quickprop_decay;                      external FANN_DLL_FILE delayed;
  procedure fann_set_quickprop_decay;                      external FANN_DLL_FILE delayed;
  function  fann_get_quickprop_mu;                         external FANN_DLL_FILE delayed;
  procedure fann_set_quickprop_mu;                         external FANN_DLL_FILE delayed;
  function  fann_get_rprop_increase_factor;                external FANN_DLL_FILE delayed;
  procedure fann_set_rprop_increase_factor;                external FANN_DLL_FILE delayed;
  function  fann_get_rprop_decrease_factor;                external FANN_DLL_FILE delayed;
  procedure fann_set_rprop_decrease_factor;                external FANN_DLL_FILE delayed;
  function  fann_get_rprop_delta_min;                      external FANN_DLL_FILE delayed;
  procedure fann_set_rprop_delta_min;                      external FANN_DLL_FILE delayed;
  function  fann_get_rprop_delta_max;                      external FANN_DLL_FILE delayed;
  procedure fann_set_rprop_delta_max;                      external FANN_DLL_FILE delayed;
  function  fann_get_rprop_delta_zero;                     external FANN_DLL_FILE delayed;
  procedure fann_set_rprop_delta_zero;                     external FANN_DLL_FILE delayed;

  // END OF DECLARATIONS OF FANN_TRAIN.H


  // DECLARATIONS OF FANN_ERROR.H

  procedure fann_set_error_log;                            external FANN_DLL_FILE delayed;
  function  fann_get_errno;                                external FANN_DLL_FILE delayed;
  procedure fann_reset_errno;                              external FANN_DLL_FILE delayed;
  procedure fann_reset_errstr;                             external FANN_DLL_FILE delayed;
  function  fann_get_errstr;                               external FANN_DLL_FILE delayed;
  procedure fann_print_error;                              external FANN_DLL_FILE delayed;

  // END OF DECLARATIONS OF FANN_ERROR

  // DECLARATIONS OF FANN_CASCADE.H

  procedure fann_cascadetrain_on_data;                     external FANN_DLL_FILE delayed;
  procedure fann_cascadetrain_on_file;                     external FANN_DLL_FILE delayed;
  function  fann_get_cascade_output_change_fraction;       external FANN_DLL_FILE delayed;
  procedure fann_set_cascade_output_change_fraction;       external FANN_DLL_FILE delayed;
  function  fann_get_cascade_output_stagnation_epochs;     external FANN_DLL_FILE delayed;
  procedure fann_set_cascade_output_stagnation_epochs;     external FANN_DLL_FILE delayed;
  function  fann_get_cascade_candidate_change_fraction;    external FANN_DLL_FILE delayed;
  procedure fann_set_cascade_candidate_change_fraction;    external FANN_DLL_FILE delayed;
  function  fann_get_cascade_candidate_stagnation_epochs;  external FANN_DLL_FILE delayed;
  procedure fann_set_cascade_candidate_stagnation_epochs;  external FANN_DLL_FILE delayed;
  function  fann_get_cascade_weight_multiplier;            external FANN_DLL_FILE delayed;
  procedure fann_set_cascade_weight_multiplier;            external FANN_DLL_FILE delayed;
  function  fann_get_cascade_candidate_limit;              external FANN_DLL_FILE delayed;
  procedure fann_set_cascade_candidate_limit;              external FANN_DLL_FILE delayed;
  function  fann_get_cascade_max_out_epochs;               external FANN_DLL_FILE delayed;
  procedure fann_set_cascade_max_out_epochs;               external FANN_DLL_FILE delayed;
  function  fann_get_cascade_max_cand_epochs;              external FANN_DLL_FILE delayed;
  procedure fann_set_cascade_max_cand_epochs;              external FANN_DLL_FILE delayed;
  function  fann_get_cascade_num_candidates;               external FANN_DLL_FILE delayed;
  function  fann_get_cascade_activation_functions_count;   external FANN_DLL_FILE delayed;
  function  fann_get_cascade_activation_functions;         external FANN_DLL_FILE delayed;
  procedure fann_set_cascade_activation_functions;         external FANN_DLL_FILE delayed;
  function  fann_get_cascade_activation_steepnesses_count; external FANN_DLL_FILE delayed;
  function  fann_get_cascade_activation_steepnesses;       external FANN_DLL_FILE delayed;
  procedure fann_set_cascade_activation_steepnesses;       external FANN_DLL_FILE delayed;
  function  fann_get_cascade_num_candidate_groups;         external FANN_DLL_FILE delayed;
  procedure fann_set_cascade_num_candidate_groups;         external FANN_DLL_FILE delayed;

end.
